import { ArrayBlock } from "../ArrayBlock";
import { Schema } from "ajv";
import { InvalidBlockError } from "../InvalidBlockError";
import { UnstableValue } from "../../NamespacedValue";
/**
 * Types for markup blocks over the wire.
 * @module Matrix Content Blocks
 * @see MarkupBlock
 */
export declare module WireMarkupBlock {
    /**
     * A representation of text within a markup block.
     * @module Matrix Content Blocks
     */
    type Representation = {
        body: string;
        mimetype?: string;
    };
}
/**
 * A "markup" block, or a block meant to communicate human-readable and human-rendered
 * text, with optional mimetype.
 * @module Matrix Content Blocks
 */
export declare class MarkupBlock extends ArrayBlock<WireMarkupBlock.Representation> {
    static readonly schema: Schema;
    static readonly validateFn: import("ajv").ValidateFunction<unknown>;
    static readonly type: UnstableValue<"m.markup", "org.matrix.msc1767.markup">;
    /**
     * Schema definition for the markup representation (list item) specifically.
     *
     * Note: Schema for the whole markup value type is handled by the ArrayBlock class.
     */
    static readonly representationSchema: Schema;
    /**
     * Validation function for markup representations (list items) specifically.
     *
     * Note: Validation for the whole markup value type is handled by the ArrayBlock class.
     */
    static readonly representationValidateFn: import("ajv").ValidateFunction<unknown>;
    /**
     * Parse errors for representations. Representations described here are *removed* from the
     * block's `raw` type, thus not being considered for rendering.
     */
    readonly representationErrors: Map<{
        index: number;
        representation: WireMarkupBlock.Representation | unknown;
    }, InvalidBlockError>;
    private lazyText;
    private lazyHtml;
    /**
     * Creates a new MarkupBlock
     *
     * Invalid representations will be removed from the `raw` value, excluding them from rendering.
     * Errors can be found from representationErrors after creating the object.
     * @param raw The block's value.
     */
    constructor(raw: WireMarkupBlock.Representation[]);
    /**
     * The text representation of the block, if one is present.
     */
    get text(): string | undefined;
    /**
     * The HTML representation of the block, if one is present.
     */
    get html(): string | undefined;
    /**
     * The ordered representations for this markup block.
     */
    get representations(): WireMarkupBlock.Representation[];
}
