"use strict";
/*
Copyright 2022 The Matrix.org Foundation C.I.C.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkupBlock = void 0;
const ArrayBlock_1 = require("../ArrayBlock");
const AjvContainer_1 = require("../../AjvContainer");
const InvalidBlockError_1 = require("../InvalidBlockError");
const NamespacedValue_1 = require("../../NamespacedValue");
const LazyValue_1 = require("../../LazyValue");
/**
 * A "markup" block, or a block meant to communicate human-readable and human-rendered
 * text, with optional mimetype.
 * @module Matrix Content Blocks
 */
class MarkupBlock extends ArrayBlock_1.ArrayBlock {
    /**
     * Creates a new MarkupBlock
     *
     * Invalid representations will be removed from the `raw` value, excluding them from rendering.
     * Errors can be found from representationErrors after creating the object.
     * @param raw The block's value.
     */
    constructor(raw) {
        super(MarkupBlock.type.stable, raw);
        /**
         * Parse errors for representations. Representations described here are *removed* from the
         * block's `raw` type, thus not being considered for rendering.
         */
        this.representationErrors = new Map();
        this.lazyText = new LazyValue_1.LazyValue(() => { var _a; return (_a = this.raw.find(m => m.mimetype === undefined || m.mimetype === "text/plain")) === null || _a === void 0 ? void 0 : _a.body; });
        this.lazyHtml = new LazyValue_1.LazyValue(() => { var _a; return (_a = this.raw.find(m => m.mimetype === "text/html")) === null || _a === void 0 ? void 0 : _a.body; });
        this.raw = raw.filter((r, i) => {
            const bool = MarkupBlock.representationValidateFn(r);
            if (!bool) {
                this.representationErrors.set({
                    index: i,
                    representation: r,
                }, new InvalidBlockError_1.InvalidBlockError(`m.markup[${i}]`, MarkupBlock.representationValidateFn.errors));
            }
            return bool;
        });
    }
    /**
     * The text representation of the block, if one is present.
     */
    get text() {
        return this.lazyText.value;
    }
    /**
     * The HTML representation of the block, if one is present.
     */
    get html() {
        return this.lazyHtml.value;
    }
    /**
     * The ordered representations for this markup block.
     */
    get representations() {
        return [...this.raw]; // clone to prevent mutation
    }
}
exports.MarkupBlock = MarkupBlock;
MarkupBlock.schema = ArrayBlock_1.ArrayBlock.schema;
MarkupBlock.validateFn = ArrayBlock_1.ArrayBlock.validateFn;
MarkupBlock.type = new NamespacedValue_1.UnstableValue("m.markup", "org.matrix.msc1767.markup");
/**
 * Schema definition for the markup representation (list item) specifically.
 *
 * Note: Schema for the whole markup value type is handled by the ArrayBlock class.
 */
MarkupBlock.representationSchema = {
    type: "object",
    properties: {
        body: {
            type: "string",
            nullable: false,
        },
        mimetype: {
            type: "string",
            nullable: false,
        },
    },
    required: ["body"],
    errorMessage: {
        properties: {
            body: "body should be a non-null string and is required",
            mimetype: "mimetype should be a non-null string, or undefined (field not required)",
        },
    },
};
/**
 * Validation function for markup representations (list items) specifically.
 *
 * Note: Validation for the whole markup value type is handled by the ArrayBlock class.
 */
MarkupBlock.representationValidateFn = AjvContainer_1.AjvContainer.ajv.compile(MarkupBlock.representationSchema);
//# sourceMappingURL=MarkupBlock.js.map