import { WireEvent } from "./types_wire";
import { RoomEvent } from "./RoomEvent";
import { NamespacedValue } from "../NamespacedValue";
/**
 * Represents an event factory for purposes of event parsing.
 * @module Event Parsing
 */
export type ParsedEventFactory<T extends object, C extends WireEvent.BlockBasedContent> = (x: WireEvent.RoomEvent<T>) => RoomEvent<C>;
/**
 * Represents a parser for unknown events, returning undefined if the event does
 * not apply. The parser should modify the `type` and `content` of the event before
 * returning the `RoomEvent<>` object, allowing consumers to see what the event got
 * interpreted as.
 *
 * Throwing InvalidBlockError and InvalidEventError are valid if the supplied event
 * *looks* compatible, but actually isn't. For example, an event containing a markup
 * block, but that markup block being invalid.
 * @module Event Parsing
 */
export type UnknownEventParser<C extends WireEvent.BlockBasedContent> = (x: WireEvent.RoomEvent<any>) => RoomEvent<C> | undefined;
/**
 * Used internally by the events-sdk to determine how to group an unknown event
 * parser.
 * @internal
 */
export declare enum InternalOrderCategorization {
    OtherMedia = 0,
    ImageMedia = 1,
    RichTextOrFile = 2,
    TextOnly = 3
}
/**
 * Add an internally-known event type to the parser. This should not be called outside
 * of the SDK itself.
 * @internal
 * @param namespace The namespace for the event.
 * @param factory The event creation factory.
 */
export declare function addInternalKnownEventParser<S extends string, U extends string, T extends object, C extends WireEvent.BlockBasedContent>(namespace: NamespacedValue<S, U>, factory: ParsedEventFactory<T, C>): void;
/**
 * Add an unknown event parser for a known event type. This should not be called outside
 * of the SDK itself.
 * @param priority The priority of this parser. See InternalOrderCategorization
 * @param parser The parser function.
 * @see InternalOrderCategorization
 */
export declare function addInternalUnknownEventParser<C extends WireEvent.BlockBasedContent>(priority: InternalOrderCategorization, parser: UnknownEventParser<C>): void;
/**
 * Parses known and unknown events to determine their validity. The SDK's internally
 * known events will always be included in the parser: there is no need to add them
 * explicitly.
 *
 * Consumers may wish to adjust the "unknown interpret order" to customize functionality
 * for when the parser encounters an event type it does not have a factory for.
 * @module Event Parsing
 */
export declare class EventParser {
    private typeMap;
    private unknownInterpretOrder;
    /**
     * All the known event types for this parser.
     */
    get knownEventTypes(): string[];
    /**
     * The default unknown event type parse order. Custom events would normally
     * get prepended to this array to ensure they get checked first.
     */
    get defaultUnknownEventParsers(): UnknownEventParser<any>[];
    /**
     * The unknown event type parsers currently known to this instance.
     */
    get unknownEventParsers(): UnknownEventParser<any>[];
    /**
     * Adds or overwrites a known type in the event parser. Does not affect the
     * "unknown event" parse order.
     * @param namespace The namespace for the event type.
     * @param factory The factory used to create the event object.
     */
    addKnownType<S extends string, U extends string, T extends object, C extends WireEvent.BlockBasedContent>(namespace: NamespacedValue<S, U>, factory: ParsedEventFactory<T, C>): void;
    /**
     * Sets the "unknown event type" parser order, where the first element of the
     * provided array will be tried first. The first matching parser will be used.
     *
     * Note that this overwrites the parsers rather than appends/prepends: callers
     * should use the defaultUnknownEventParsers property to prepend their custom
     * events prior to calling this set function.
     * @param ordered The parsers, ordered by which parser should be tried first.
     * @see defaultUnknownEventParsers
     */
    setUnknownParsers<C extends WireEvent.BlockBasedContent = WireEvent.BlockBasedContent>(ordered: UnknownEventParser<C>[]): void;
    /**
     * Parses an event. If the event type is known to the parser, it will be parsed
     * as that type. If the event type is unknown, the "unknown event" parsers will
     * be used to return an appropriate type. If no parser applies after both sets
     * are checked, undefined is returned.
     *
     * An event returned by this function might have different attributes (event type,
     * content, etc) than the event supplied in the function: this is to allow callers
     * to check what type of event the parser is treating given type as. For example,
     * if an `org.example.sample` event type was given here, the function might return
     * an event type which matches `m.message`.
     *
     * Note that this function can throw InvalidBlockError and InvalidEventError, or
     * other errors, if the given event is unforgivably unable to be parsed.
     * @param event The event to try parsing.
     * @returns The parsed event, or undefined if unable.
     */
    parse(event: WireEvent.RoomEvent): RoomEvent | undefined;
}
