import { WireEvent } from "./types_wire";
import { Schema } from "ajv";
/**
 * Minimum representation of a room (timeline or state) event in Matrix, for
 * consumption by a parser.
 * @module Events
 */
export declare abstract class RoomEvent<Content extends WireEvent.BlockBasedContent = WireEvent.BlockSpecificContent<{}>> {
    readonly name: string;
    readonly raw: WireEvent.RoomEvent<Content>;
    static readonly schema: Schema;
    static readonly validateFn: import("ajv").ValidateFunction<unknown>;
    /**
     * Creates a new MatrixEvent, validating the event object itself. Implementations of
     * this abstract class should only need to validate the content object rather than the
     * whole event schema.
     * @param name The name of the event. Used for debugging.
     * @param raw The raw event itself.
     * @param isStateEvent True (default: false) if the event type is expected to be a
     * state event. The event will be strictly checked to ensure compliance with this
     * field.
     * @protected
     */
    protected constructor(name: string, raw: WireEvent.RoomEvent<Content>, isStateEvent?: boolean);
    /**
     * The room ID this event was sent in.
     */
    get roomId(): string;
    /**
     * The event ID this event was sent as.
     */
    get eventId(): string;
    /**
     * The raw, unparsed, content of this event. It is recommended to use the getters
     * on the event object instead to retrieve relevant parts of the event, such as
     * message text or image details.
     */
    get content(): Content;
    /**
     * The type of this event.
     */
    get type(): string;
    /**
     * The sender (user ID) of this event.
     */
    get sender(): string;
    /**
     * The state key for this event, if present. Note that an empty string is a
     * valid state key: check for undefined to determine presence of a state key.
     */
    get stateKey(): string | undefined;
    /**
     * The reported timestamp this event was sent at. Note that this is supplied
     * by the sender, and will be zero if negative.
     */
    get timestamp(): number;
}
